__author__ = "songjiangshan"
__copyright__ = "Copyright (C) 2021 songjiangshan \n All Rights Reserved."
__license__ = ""
__version__ = "1.0"

from django.contrib import admin
from django.urls import path, re_path
from django.urls import path, include
from django.views.generic.base import RedirectView

from django.urls import re_path, path, include
from rest_framework import routers
from .update_rest import update_rest


from .my_rest_api import my_rest_viewsetB
import json
router = routers.DefaultRouter()
#from .urls_rest import *

try:
    from .urls_rest import *
except Exception as e:
    print('ERROR: django_rest_admin error. rest may not not work as expected.')
    print(e)

def default_list(request):
    return ''

urlpatterns = [
    path('api/', include(router.urls), name='django_rest_admin_api'),
    path('default_list/', default_list, name='django_rest_admin_default_list'),
]




# coding:utf-8

__author__ = "songjiangshan"
__copyright__ = "Copyright (C) 2021 songjiangshan \n All Rights Reserved."
__license__ = ""
__version__ = "1.0"

import io
from .models import RouteExec, ComputedField
import json
from django.http import HttpResponse



def parse_params(params_str):
    ret_dict = {}
    if params_str is None:
        return ret_dict
    if len(params_str) == 0:
        return ret_dict

    if isinstance(params_str, dict):
        return params_str
    try:
        ret_dict = json.loads(params_str)
    except Exception as e:
        print(e)
        print('param parse error')
        print(params_str)

    return ret_dict


def foreign_key_gen(table_name, related_name, del_type='CASCADE'):
    """
    del_type: CASCADE SET_NULL DO_NOTHING

    """

    return "models.ForeignKey(to=" + table_name + ", db_constraint=False, on_delete=models."+del_type+", blank=True, null=True, related_name='" + related_name + "')"

def str_to_obj(obj):
    if obj is None:
        return obj
    return json.loads(obj)


def params_foreign_key_id_update(one_r):
    if one_r['foreign_key_id'] is None:
        return one_r
    if one_r['foreign_key_id'] == '':
        return one_r
    fk = str_to_obj(one_r['foreign_key_id'])
    need_save = 0
    for i in fk:
        if isinstance(fk[i], str):
            fk[i] = [fk[i], 'CASCADE']
            need_save = 1

    if need_save:
        one_r['foreign_key_id'] = json.dumps(fk, indent=2)

    return one_r


def params_update_list(all_one_list):
    all_one_list2=[]
    for i in all_one_list:
        i=params_updata(i)
        all_one_list2.append(i)
    return all_one_list2


def params_updata(one_r: dict):
    one_r = params_foreign_key_id_update(one_r)

    params = {
        'foreign_key_id': str_to_obj(one_r['foreign_key_id']),
        'foreign_key_ro': str_to_obj(one_r['foreign_key_ro']),
        'foreign_slug_kf': str_to_obj(one_r['foreign_slug_kf']),
        'ordering_fields': str_to_obj(one_r['ordering_fields']),
        'ordering': str_to_obj(one_r['ordering']),
        'no_need_login': one_r['no_need_login'],
        'search_fields': str_to_obj(one_r['search_fields']),
        'filter_keys': str_to_obj(one_r['filter_keys']),
        'model_object_list': str_to_obj(one_r['model_object_list'])
    }

    for i in list(params.keys()):
        if params[i] is None:
            del params[i]
    one_r['params'] = json.dumps(params)
    return one_r





def update_models(all_rest_dict_list):
    """
    更新模型文件
    1 读取routeexec种的表名，使用命令inspected生成默认表结构
    2 替换foreign_key 字段
    3 去掉id字段。id如果存在，会导致django程序错误。针对部分表导出后有id的问题
    """
    from django.core.management import call_command
    from django.conf import settings
    import os

    path1 = os.path.dirname(__file__)
    file_name = os.path.join(path1,'models_inspected.py')

    print(file_name)

    #保存inspectdb数据到stringio.
    f = io.StringIO()

    f.write("from django.contrib.auth.models import User\n")
    #f.write("from AmisNavigationBar.models import AmisNavigation\n")

    table_list = []
    for i in all_rest_dict_list:
        if i['inspected_from_db'] !=1:
            continue
        table_list.append(i['table_name'])
    if len(table_list) > 0:
        call_command("inspectdb", table_list, stdout=f)
    else:
        f.write('#no table exist in route_exec\n')
    # 所有数据读出到字符串变量：models_new
    models_new = f.getvalue()
    f.close()


    # 此变量用户下面的model外键更改
    # key:className
    # value:{ foreign_key_id中的key _id : foreign_key_id_value  }
    foreign_key_dict2 = {}
    for one_r in all_rest_dict_list:
        # if one_r.re_type == 'table':
        #one_r = params_updata(one_r)
        params = parse_params(one_r['params'])
        if one_r['inspected_from_db'] != 1:
            continue
        if 'foreign_key_id' in params:
            foreign_key_dict2[one_r['table_big_name']] = {}
            for k in params['foreign_key_id']:
                foreign_key_dict2[one_r['table_big_name']][k + '_id'] = params['foreign_key_id'][k]

    print('update_models foreign_key_dict2:', foreign_key_dict2)

    # 当前model名
    curr_class_name = ''
    f2 = open(file_name, 'w+')
    for one_line in models_new.split('\n'):
        #每行分析处理
        if len(one_line.strip()) == 0:
            # 空行
            f2.write(one_line + "\n")
            continue
        one_line_start_space = len(one_line) - len(one_line.lstrip())
        one_line_striped = one_line.strip()
        if one_line_striped[0] == '#':
            # 注释
            f2.write(one_line + "\n")
            continue

        spt = one_line_striped.split(' ')
        if len(spt) == 0:
            # 没有空格，不认识的行??
            f2.write(one_line + "\n")
            continue

        if (one_line_start_space==0) and (spt[0] == 'class'):
            # 获取类名
            curr_class_name = spt[1].split('(')[0]
            f2.write(one_line + "\n")
            continue

        elif spt[0] == 'class':
            #内部类，在内部类之前，先放入计算属性值
            re1 = RouteExec.objects.filter(table_big_name=curr_class_name, inspected_from_db=1).all()
            if len(re1)<1:
                print('skip this table',curr_class_name, len(re1))
            cf = ComputedField.objects.filter(route_exec=re1[0]).all()
            for cfi in cf:
                f2.write(' ' * one_line_start_space+cfi.func_text.replace('\r\n','\n'))
                f2.write(' ' * one_line_start_space+"\n")

            f2.write(one_line + "\n")
            continue

        curr_field_name = spt[0]
        if curr_field_name=='id':
            #id自动去除，避免djanog错误
            f2.write(' ' * one_line_start_space )
            f2.write('#' + one_line)
            f2.write('\n')
            continue


        # managed处理
        if curr_field_name == 'managed' and len(spt) == 3 and spt[2] != 'True':
            # 不是需要的字段，直接复制
            f2.write(' ' * one_line_start_space + "managed = True\n")
            continue

        if curr_class_name not in foreign_key_dict2:
            # 当前class没有外键
            f2.write(one_line + "\n")
            continue

        # 当前class的所有外键
        foreign_key_dict = foreign_key_dict2[curr_class_name]

        # 当前字段是外键
        if curr_field_name in foreign_key_dict:
            # 此行要替换为外键代码

            f2.write(' ' * one_line_start_space)
            f2.write(curr_field_name.replace('_id', ''))
            f2.write(' = ')
            print(curr_class_name, curr_field_name, foreign_key_dict[curr_field_name][0])
            if foreign_key_dict[curr_field_name][0] == curr_class_name:
                itable_name = '"self"'
            else:
                itable_name = foreign_key_dict[curr_field_name][0]

            irelated_name = curr_class_name +'_'+ curr_field_name.replace('_id', '')
            print('irelated_name', irelated_name)
            idel_typ = foreign_key_dict[curr_field_name][1]
            f2.write(foreign_key_gen(itable_name, irelated_name, idel_typ))
            f2.write('\n')
            continue


        f2.write(one_line + "\n")
        continue

    # model写完。此处添加receiver
    path1 = os.path.dirname(__file__)
    file_name_receiver = os.path.join(path1,'models_receiver.py')
    f_recv = open(file_name_receiver, 'r')
    f2.write(f_recv.read())

    f2.close()

    return 'ok'



__author__ = "songjiangshan"
__copyright__ = "Copyright (C) 2021 songjiangshan \n All Rights Reserved."
__license__ = ""
__version__ = "1.0"

import json
from django.shortcuts import render
from rest_framework.response import Response
from rest_framework import viewsets
from rest_framework import serializers
from rest_framework import routers
from django_filters import rest_framework as filters
from rest_framework import filters as rest_framework_filter
import django_filters
import django_filters.rest_framework


'''
    封装 对表的增删改查 rest_api
'''



# curr_view = my_rest_apiB(PickNumber, 'PickNumber', ["id", 'team_name', 'pick_number', 'team'], {'team_name':'team.name'}, {'team':Team})
def my_rest_viewsetB(model_object,model_class_name, model_obj_list='__all__', foreign_key_ro={}, foreign_key_id={}, filter_fieldsA=(),
                     no_need_login = False, optional_fields1=[], search_fieldsA=[], orderingA=[], ordering_fieldsA=[], filter_keys=[], foreign_slug_kf={}):
    """

    :param model_object: django的数据模型
    :param model_class_name:数据模型的名称，用于生成代码名称，一般是model_object的字符串
    :param model_obj_list: 可查看的字段列表['id','name']. 可以是 '__all__'代表所有字段
    :param foreign_key_ro:只读外键内容：{'team_name': 'team.name', }
    :param foreign_key_id: 只读外键类型{'team':Team}
    :param filter_fields: 可搜索字段 ['id','team','name']
    :param no_need_login: 是否需要登录认证
    :param optional_fields1: 字段是否是可选，用于数据库写入设置
    :return:
    """
    #print('my_rest_viewsetB', model_obj_list)
    def add_foreign_serializer(ModName, fields_list):
        class TrackSerializer(serializers.ModelSerializer):
            class Meta:
                model = globals()[ModName]
                fields = fields_list

        TrackSerializer.__name__=ModName+"_in_"+model_class_name
        return TrackSerializer

    def AddVarFunc(self, foreign_key_ro, foreign_key_id, foreign_slug_kf):
        if foreign_key_ro is not None:
            for i in foreign_key_ro:
                #setattr(self, i, serializers.CharField(source=foreign_key_ro[i], read_only=True))
                self[i]=serializers.CharField(source=foreign_key_ro[i], read_only=True)
        if foreign_key_id is not None:
            for i in foreign_key_id:
                #setattr(self, i,
                #        serializers.PrimaryKeyRelatedField(queryset=foreign_key_id[i].objects.all(), read_only=False))
                self[i]=serializers.PrimaryKeyRelatedField(queryset=foreign_key_id[i].objects.all(), read_only=False, required=False, allow_null=True)

        if foreign_slug_kf is not None:
            for i in foreign_slug_kf:
                self[i] = add_foreign_serializer(i.split('_')[0], foreign_slug_kf[i])(many=True, read_only=True)
                #self[i] = serializers.SlugRelatedField( many=True,
                #                        read_only=True,
                #                        slug_field=foreign_slug_kf[i])

        return 'A'

    class TeamPickNumberSerializer(serializers.ModelSerializer):
        useless_cc = AddVarFunc(locals(), foreign_key_ro, foreign_key_id, foreign_slug_kf)

        #team_name = serializers.CharField(source='team.name', read_only=True)  # team_name 用于页面显示文字
        #team = serializers.PrimaryKeyRelatedField(queryset=model_object.objects.all(), read_only=False)  # team_id  用于修改数据库
        class Meta:
            model = model_object
            fields = model_obj_list
            #fields = '__all__'
            extra_kwargs = {i: {"required": False, "allow_null": True} for i in optional_fields1}

    class GoodsFilter(django_filters.rest_framework.FilterSet):
        if filter_keys is not None:
            for i in filter_keys:
                if ('filter_type' not in i) or ('filter_name' not in i) or ('field_name' not in i) or (
                        'lookup_expr' not in i):
                    print('WARN:filter no type will not effect', i, model_class_name)
                    continue
                if i['filter_type'] == 'number':
                    locals()[i['filter_name']] = django_filters.NumberFilter(field_name=i['field_name'], lookup_expr=i['lookup_expr'])
                elif i['filter_type'] == 'text':
                    locals()[i['filter_name']] = django_filters.CharFilter(field_name=i['field_name'], lookup_expr=i['lookup_expr'])
                elif i['filter_type'] == 'bool':
                    locals()[i['filter_name']] = django_filters.BooleanFilter(field_name=i['field_name'], lookup_expr=i['lookup_expr'])
                elif i['filter_type'] == 'date_range':
                    locals()[i['filter_name']] = django_filters.DateFromToRangeFilter(field_name=i['field_name'], lookup_expr='range')
                elif i['filter_type'] == 'time_range':
                    locals()[i['filter_name']] = django_filters.TimeRangeFilter(field_name=i['field_name'],
                                                                                  lookup_expr='range')
                elif i['filter_type'] == 'date':
                    locals()[i['filter_name']] = django_filters.DateFilter(field_name=i['field_name'], lookup_expr=i['lookup_expr'])
                elif i['filter_type'] == 'time':
                    locals()[i['filter_name']] = django_filters.TimeFilter(field_name=i['field_name'], lookup_expr=i['lookup_expr'])
                elif i['filter_type'] == 'datetime':
                    locals()[i['filter_name']] = django_filters.DateTimeFilter(field_name=i['field_name'], lookup_expr=i['lookup_expr'])
                elif i['filter_type'] == 'isodatetime':
                    locals()[i['filter_name']] = django_filters.IsoDateTimeFilter(field_name=i['field_name'],
                                                                           lookup_expr=i['lookup_expr'])
                else:
                    print('WARN: unknown filter type:', i['filter_type'], model_class_name, i)

        class Meta:
            model = model_object
            fields = [ (fk1['filter_name'] for fk1 in filter_keys) if filter_keys is not None else 'id']

    GoodsFilter.__name__= model_class_name+'SFilter'


    class TeamPickNumberView(viewsets.ModelViewSet):
        filterset_class = GoodsFilter
        #filter_fields = filter_fieldsA

        queryset = model_object.objects.all()
        serializer_class = TeamPickNumberSerializer
        filter_backends = [django_filters.rest_framework.DjangoFilterBackend,rest_framework_filter.SearchFilter, rest_framework_filter.OrderingFilter]#,rest_framework_filter.SearchFilter, rest_framework_filter.OrderingFilter)
        search_fields = search_fieldsA
        ordering = orderingA
        ordering_fields = ordering_fieldsA

    if no_need_login:
        # print('no_need_login for:', model_class_name)
        TeamPickNumberView.authentication_classes = ()
        TeamPickNumberView.permission_classes = ()
    TeamPickNumberView.__name__= model_class_name+'View'
    TeamPickNumberSerializer.__name__ = model_class_name+'Serial'
    return TeamPickNumberView




from rest_framework.renderers import JSONRenderer
class EmberJSONRenderer(JSONRenderer):
    def render(self, data, accepted_media_type=None, renderer_context=None):
        data = {'status':0,'msg':'',  'data':{ 'items': data}}
        return super(EmberJSONRenderer, self).render(data, accepted_media_type, renderer_context)







# Generated by Django 3.2.8 on 2021-10-28 14:47

from django.db import migrations, models
import django.db.models.deletion
import django_rest_admin.model_field


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='RouteExec',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('route', models.TextField(blank=True, help_text='the route name. eg: /Danwei', null=True)),
                ('table_big_name', models.TextField(blank=True, help_text='the model name of a table. eg: Danwei', null=True)),
                ('inspected_from_db', models.IntegerField(blank=True, help_text='是否需要将库中表导入为django的model。1，0. eg: 0 ', null=True)),
                ('table_name', models.TextField(blank=True, help_text='the table name. eg: danwei. ONLY needed if inspected_from_db=1', null=True)),
                ('import_py_code', models.TextField(blank=True, help_text='needed if the model need to import from somewhere else. eg:from django.auth import User', null=True)),
                ('desc', models.TextField(blank=True, help_text='表功能描述. eg: this is the rest description', null=True)),
                ('foreign_key_id', django_rest_admin.model_field.JSONField(blank=True, help_text='外键定义.关联删除属性：CASCADE-关联删除, SET_NULL-字段变空, DO_NOTHING-不做操作. eg: {"user":["User", "CASCADE"], "article":["Article", "SET_NULL"]}', null=True)),
                ('foreign_key_ro', django_rest_admin.model_field.JSONField(blank=True, help_text='外键关联的只读内容. eg: {"article_name":"article.name"}', null=True)),
                ('foreign_slug_kf', django_rest_admin.model_field.JSONField(blank=True, help_text='一对多外键关联的字段列表. eg: {"StatKucun_wuliao": ["kucun", "jinjia"]}', null=True)),
                ('ordering_fields', django_rest_admin.model_field.JSONField(blank=True, help_text='可用排序字段，数组，每个项是字符串. eg:["id", "name"]', null=True)),
                ('ordering', django_rest_admin.model_field.JSONField(blank=True, help_text='默认排序字段，数组，每个项是字符串. eg:["id", "name"]', null=True)),
                ('no_need_login', models.IntegerField(blank=True, help_text='访问是否需要登陆。1，0. eg: 0 ', null=True)),
                ('search_fields', django_rest_admin.model_field.JSONField(blank=True, help_text='可用排序字段，数组，每个项是字符串. eg: ["name", "zhujima"]', null=True)),
                ('filter_keys', django_rest_admin.model_field.JSONField(blank=True, help_text='过滤项. eg: [{"filter_name": "name", "field_name": "name", "filter_type": "text", "lookup_expr": "icontains"}]', null=True)),
                ('model_object_list', django_rest_admin.model_field.JSONField(blank=True, help_text='可显示字段. eg: ["id", "name", "daima"]', null=True)),
                ('params', django_rest_admin.model_field.JSONField(blank=True, help_text='AUTO-GENERATED FIELD. DO NOT EDIT THIS MANUALLY.', null=True)),
            ],
            options={
                'verbose_name_plural': 'table-REST-CRUD',
                'db_table': 'rest_admin_table_crud',
                'managed': True,
            },
        ),
        migrations.CreateModel(
            name='ComputedField',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('func_text', django_rest_admin.model_field.CodeField(blank=True, help_text='py函数.eg: def jine(self):return 0', null=True)),
                ('route_exec', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_rest_admin.routeexec')),
            ],
            options={
                'verbose_name_plural': '计算函数字段',
                'db_table': 'rest_admin_table_crud_computed_field',
                'managed': True,
            },
        ),
    ]



from django.db import models
from .model_field import JSONField,CodeField

# Create your models here.


class RouteExec(models.Model):
    route = models.TextField(blank=True, null=True, help_text='the route name. eg: /Danwei')
    table_big_name = models.TextField(blank=True, null=True, help_text='the model name of a table. eg: Danwei')
    inspected_from_db = models.IntegerField(blank=True, null=True, help_text='是否需要将库中表导入为django的model。1，0. eg: 0 ')
    table_name = models.TextField(blank=True, null=True,
                                  help_text='the table name. eg: danwei. ONLY needed if inspected_from_db=1')

    import_py_code = models.TextField(blank=True, null=True, help_text='needed if the model need to import from somewhere else. eg:from django.auth import User')
    desc = models.TextField(blank=True, null=True, help_text="表功能描述. eg: this is the rest description")
    foreign_key_id= JSONField(blank=True, null=True, help_text='外键定义.关联删除属性：CASCADE-关联删除, SET_NULL-字段变空, DO_NOTHING-不做操作. eg: {"user":["User", "CASCADE"], "article":["Article", "SET_NULL"]}')
    foreign_key_ro= JSONField(blank=True, null=True, help_text='外键关联的只读内容. eg: {"article_name":"article.name"}')
    foreign_slug_kf =JSONField(blank=True, null=True, help_text='一对多外键关联的字段列表. eg: {"StatKucun_wuliao": ["kucun", "jinjia"]}')
    ordering_fields = JSONField(blank=True, null=True, help_text='可用排序字段，数组，每个项是字符串. eg:["id", "name"]')
    ordering  = JSONField(blank=True, null=True, help_text='默认排序字段，数组，每个项是字符串. eg:["id", "name"]')
    no_need_login = models.IntegerField(blank=True, null=True, help_text='访问是否需要登陆。1，0. eg: 0 ')
    search_fields = JSONField(blank=True, null=True, help_text='可用排序字段，数组，每个项是字符串. eg: ["name", "zhujima"]')
    filter_keys= JSONField(blank=True, null=True, help_text='过滤项. eg: [{"filter_name": "name", "field_name": "name", "filter_type": "text", "lookup_expr": "icontains"}]')
    model_object_list = JSONField(blank=True, null=True, help_text='可显示字段. eg: ["id", "name", "daima"]')
    params = JSONField(blank=True, null=True, help_text='AUTO-GENERATED FIELD. DO NOT EDIT THIS MANUALLY.')



    class Meta:
        managed = True
        verbose_name_plural = 'table-REST-CRUD'
        db_table = 'rest_admin_table_crud'

class ComputedField(models.Model):
    route_exec = models.ForeignKey(RouteExec, on_delete=models.CASCADE)
    func_text = CodeField(blank=True, null=True, help_text="py函数.eg: def jine(self):return 0")
    class Meta:
        managed = True
        verbose_name_plural = '计算函数字段'
        db_table = 'rest_admin_table_crud_computed_field'



from django.contrib import admin
from .models import RouteExec,ComputedField
from django.contrib import admin
from django.utils.html import format_html
from django.urls import path, include
from django.http import HttpResponse,HttpResponseRedirect

from .update_models import update_models
from .json_widget import JsonEditorWidget, CodeEditorWidget
from .model_field import JSONField, CodeField
from .update_rest import update_rest
from .get_app_url_base import get_app_url_base

# Register your models here.



class ComputedFieldAdmin(admin.TabularInline):
    model = ComputedField

class RouteExecAdmin(admin.ModelAdmin):
    inlines = [ComputedFieldAdmin, ]
    change_list_template = "html/auto_refresh_list.html"

    def get_urls(self):
        urls = super().get_urls()
        my_urls = [
            path('update_rest/', self.update_rest_action),
        ]
        return my_urls + urls
    def update_rest_action(self, request):
        update_rest(request)
        return HttpResponseRedirect("../")



    def button_link(self, obj):
        app_base = get_app_url_base()
        button_html = """<a class="changelink" href=""" + app_base+ """/api%s/>OpenApi</a>"""%(obj.route)
        return format_html(button_html)

    button_link.short_description = "打开"

    list_display = ['id', 'route', 'button_link']
    formfield_overrides = {
            JSONField: {'widget': JsonEditorWidget},
            CodeField: {'widget': CodeEditorWidget},
        }

    class Media:
        css = {
            'all': ( 'django_rest_admin/jsoneditor.css',)
        }
        js = ('django_rest_admin/jsoneditor.js', 'django_rest_admin/jquery-3.6.0.min.js')


admin.site.register(RouteExec, RouteExecAdmin)





# coding:utf-8

__author__ = "songjiangshan"
__copyright__ = "Copyright (C) 2021 songjiangshan \n All Rights Reserved."
__license__ = ""
__version__ = "1.0"

import json
from django.db import models


class JSONField(models.TextField):
    #__metaclass__ = models.SubfieldBase
    description = "this is a json Text"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def db_type(self,connection):
        return super().db_type(connection)
    def to_python(self, value):
        v = models.TextField.to_python(self, value)
        try:
            return json.loads(v)
        except:
            return '{}'
        return v
    def get_prep_value(self, value):
        if value is None:
            return None
        if isinstance(value, str):
            return value
        return json.dumps(value)



class CodeField(models.TextField):
    #__metaclass__ = models.SubfieldBase
    description = "this is a code Text"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def db_type(self,connection):
        return super().db_type(connection)

    #
    #def from_db_value(self, value, expression, connection):
    #    #ret = super().from_db_value(value, expression, connection)
    #    return self.to_python(value)


    def to_python(self, value):
        v = models.TextField.to_python(self, value)
        return v
    #
    def get_prep_value(self, value):
        if value is None:
            return None
        if isinstance(value, str):
            return value
        return str(value)




__author__ = "songjiangshan"
__copyright__ = "Copyright (C) 2021 songjiangshan \n All Rights Reserved."
__license__ = ""
__version__ = "1.0"

from django.urls import reverse

def get_app_url_base():
    front_path = reverse('django_rest_admin_default_list')
    if front_path[-1]=='/':
        front_path=front_path[:-1]

    pos = front_path.rfind('/')
    front_path=front_path[:pos]
    return front_path




# coding: utf-8

import setuptools

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()
	
setuptools.setup(
    name="django_rest_admin",
    version="0.0.8",
    author="jiangshan00000",
    author_email="710806594@qq.com",
    description="django app for Adding table CRUD rest api with admin ui and without coding.",
	long_description=long_description,
	long_description_content_type = "text/markdown",
    url="https://github.com/Jiangshan00001/django_rest_admin",
    packages=setuptools.find_packages(),
    include_package_data=True,

    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
	install_requires=[
    'django',
    'django-filter',
	'djangorestframework',
	],
)


__author__ = "songjiangshan"
__copyright__ = "Copyright (C) 2021 songjiangshan \n All Rights Reserved."
__license__ = ""
__version__ = "1.0"

import os
import json
from django.http import HttpResponse
from django.forms.models import model_to_dict
from .models import RouteExec, ComputedField
from .update_models import update_models,params_update_list,parse_params



def resort_rest_models(all_rest_dict_list):
    all_rest_dict_list_new=[]

    # 先按照inspected和id排序
    all_rest_dict_list.sort(key=lambda x:( 0 if x['inspected_from_db'] is None else x['inspected_from_db'], x['id'] ) )

    #重新设置每个的id
    index=0
    for i in all_rest_dict_list:
        i['id'] =index
        index+=100

    # 排序前，准备好各种数据：
    # model_dict={table_big_name:table_dict,...} 获得每个table_big_name对应的id
    # table_dict['foreign_key_models']=[table_big_name1,...] 依赖关系
    model_dict={}
    for i in all_rest_dict_list:
        model_dict[i['table_big_name']] = i

        i['foreign_key_models'] = []

        if i['inspected_from_db']!=1:
            continue
        if (i['foreign_key_id'] is None) or (i['foreign_key_id'] =='') or (i['foreign_key_id'] =='{}') or (i['foreign_key_id'] =='[]'):
            continue

        fk = json.loads(i['foreign_key_id'])
        for j in fk:
            i['foreign_key_models'].append(fk[j][0])


    # 根据model_dict 和 table_dict进行排序，顺序由id标识
    for i in all_rest_dict_list:
        i['foreign_key_models_smallest'] = i['id']
        for j in i['foreign_key_models']:
            if j in model_dict:
                if i['foreign_key_models_smallest']<model_dict[j]['id']:
                    i['foreign_key_models_smallest'] = model_dict[j]['id']+1
        if i['foreign_key_models_smallest']>i['id']:
            i['id'] = i['foreign_key_models_smallest']



    all_rest_dict_list.sort(key=lambda x:(  x['id'] ) )

    for i in all_rest_dict_list:

        print(i['id'],  i['table_big_name'], i['foreign_key_models_smallest'])

    return all_rest_dict_list

def write_to_file(to_write_str):

    path1 = os.path.dirname(__file__)
    print('write_to_file-path1:',path1)
    urls_rest_py_file_path = os.path.join(path1,'urls_rest.py')
    print('write_to_file:',urls_rest_py_file_path)
    f_to_w = open(urls_rest_py_file_path,'wb')
    f_to_w.write(to_write_str.encode('utf-8'))
    f_to_w.close()

def list_model_to_dict(all_rest):
    all_rest_dict_list=[]
    for i in all_rest:
        all_rest_dict_list.append(model_to_dict(i))
    return all_rest_dict_list


def none_str(kk):
    return 'None' if kk is None else str(kk)

def generate_rest_code(all_rest_dict_list):
    to_write_str = 'from .models_inspected import *\n'
    to_write_str +='from .my_rest_api import my_rest_viewsetB\n'
    to_write_str += 'from .urls import router\n'

    for i in all_rest_dict_list:
        to_write_str+='####################################\n'
        to_write_str += '#for route'+i['route']+'\n'
        to_write_str += i['import_py_code']+'\n'
        to_write_str += 'routeName="' + i['route']+'"\n'
        to_write_str += """
if routeName[0] == '/':
    routeName = routeName[1:]
tableBName = routeName
"""
        to_write_str +='foreign_key_ro = ' +none_str(i['foreign_key_ro'])+'\n'
        to_write_str += 'foreign_key_id = ' + none_str(i['foreign_key_id']) + '\n'
        to_write_str += 'model_obj_list = ' + none_str(i['model_object_list']) + '\n'
        to_write_str += 'filter_fields = None\n' #+ none_str(i['filter_fields']) + '\n'
        to_write_str += 'no_need_login = ' + none_str(i['no_need_login']) + '\n'
        to_write_str += 'search_fieldsA = ' + none_str(i['search_fields']) + '\n'
        to_write_str += 'ordering = ' + none_str(i['ordering']) + '\n'
        to_write_str += 'ordering_fields = ' + none_str(i['ordering_fields']) + '\n'
        to_write_str += 'filter_keys = ' + none_str(i['filter_keys']) + '\n'
        to_write_str += 'foreign_slug_kf = ' + none_str(i['foreign_slug_kf']) + '\n'

        to_write_str += 'if model_obj_list is None:\n    model_obj_list="__all__"\n'

        to_write_str += 'if foreign_key_id is not None:\n    for i in foreign_key_id:\n        foreign_key_id[i]= globals()[foreign_key_id[i][0]]\n'

        to_write_str +=  "choice_problems = my_rest_viewsetB(" + str(i['table_big_name']) + ", tableBName + 'V',"
        to_write_str += """

                                               model_obj_list=model_obj_list, no_need_login=no_need_login,
                                               foreign_key_ro=foreign_key_ro, foreign_key_id=foreign_key_id,
                                               filter_fieldsA=filter_fields,
                                               search_fieldsA=search_fieldsA, orderingA=ordering,
                                               ordering_fieldsA=ordering_fields, filter_keys=filter_keys,
                                               foreign_slug_kf=foreign_slug_kf)
                                               \n
                                               """
        to_write_str += '\nrouter.register(routeName, choice_problems) \n'
        to_write_str += '####################################\n'


    return to_write_str




def update_rest(request):
    """
    将各rest的依赖关系顺序整理好
    依次输出代码至 urls_rest
    """
    all_rest = RouteExec.objects.all()
    all_rest_dict_list=list_model_to_dict(all_rest)

    all_rest_dict_list = params_update_list(all_rest_dict_list)

    all_rest_dict_list = resort_rest_models(all_rest_dict_list)

    update_models(all_rest_dict_list)

    to_write_str=generate_rest_code(all_rest_dict_list)
    write_to_file(to_write_str)


    return HttpResponse('generate')



# coding:utf-8

__author__ = "songjiangshan"
__copyright__ = "Copyright (C) 2021 songjiangshan \n All Rights Reserved."
__license__ = ""
__version__ = "1.0"

import json

from django.forms import Widget, widgets
from django.utils.safestring import mark_safe

widgets.TextInput

class JsonEditorWidget(Widget):

    """
    在 django  admin 后台中使用  jsoneditor 处理 JSONField

    TODO：有待改进, 这里使用 % 格式化，使用 format 会抛出 KeyError 异常
        """

    html_template = """
        <div id='%(name)s_editor_holder' style='padding-left:170px'></div>
    
        <textarea hidden readonly class="vLargeTextField" cols="40" id="id_%(name)s" name="%(name)s" rows="20">%(value)s</textarea>
    
    <style type="text/css">
    .jsoneditor-mode-code {
      height: 500px;
    }
  </style>

    
        <script type="text/javascript">
            var element = document.getElementById('%(name)s_editor_holder');
            var json_value = %(value)s;
    
            var %(name)s_editor = new JSONEditor(element, {
                onChange: function() {
                    var textarea = document.getElementById('id_%(name)s');
                    var json_changed = JSON.stringify(%(name)s_editor.get());
                    textarea.value = json_changed;
                },
                mode: 'code',
                modes: ['code', 'form', 'text', 'tree', 'view', 'preview'], // allowed modes
            });
    
            %(name)s_editor.set(json_value)
            %(name)s_editor.expandAll()
        </script>
        """


    def __init__(self, attrs=None):
        super().__init__(attrs)


    def render(self, name, value, attrs=None, renderer=None):
        #print('JsonEditorWidget render')
        if isinstance(value, str):
            value = json.loads(value)
        elif isinstance(value, dict):
            value=value

        if renderer is None:
            renderer = self.get_default_renderer()

        #return mark_safe(renderer.render(template_name, context))


        result = self.html_template % {'name': name, 'value': json.dumps(value), }
        return mark_safe(result)


class CodeEditorWidget(Widget):
    """
    在 django  admin 后台中使用  jsoneditor 处理 JSONField

    TODO：有待改进, 这里使用 % 格式化，使用 format 会抛出 KeyError 异常
        """

    html_template = """
        <div id='%(name)s_editor_holder' style='padding-left:170px'></div>

        <textarea hidden readonly class="vLargeTextField" cols="40" id="id_%(name)s" name="%(name)s" rows="20">%(value)s</textarea>

    <style type="text/css">
    .jsoneditor-mode-code {
      height: 500px;
    }
  </style>


        <script type="text/javascript">
            var element = document.getElementById('%(name)s_editor_holder');
            var json_value = %(value)s;

            var %(name)s_editor = new JSONEditor(element, {
                onChange: function() {
                    var textarea = document.getElementById('id_%(name)s');
                    var json_changed = %(name)s_editor.get();
                    textarea.value = json_changed;
                },
                mode: 'code',
                modes: ['code', 'form', 'text', 'tree', 'view', 'preview'], // allowed modes
            });

            %(name)s_editor.set(json_value)
            %(name)s_editor.expandAll()
        </script>
        """

    def __init__(self, attrs=None):
        super().__init__(attrs)

    def render(self, name, value, attrs=None, renderer=None):
        # print('JsonEditorWidget render')
        #value = value

        if renderer is None:
            renderer = self.get_default_renderer()

        # return mark_safe(renderer.render(template_name, context))

        result = self.html_template % {'name': name, 'value': value, }
        return mark_safe(result)
